#ifndef List_H
#define List_H

template <class T>
class ListItem {
private:
    T itemVal;
    ListItem<T> * next;

public:

    ListItem(T val) {
        itemVal = val;
        next = NULL;
    }

    T getVal() {
        return itemVal;
    }

    void setNext(ListItem<T> * n) {
        next = n;
    }

    ListItem<T> * getNext() {
        return next;
    }
};

template <class T>
class List {
private:
    ListItem<T> * head;
    ListItem<T> * tail;
    int count;

public:

    List() {
        head = NULL;
        tail = NULL;
        count = 0;
    }

    ~List() {
        clear();
    }

    bool getHead(T & val) {
        if (head == NULL) {
            return false;
        }
        val = head->getVal();
        return true;
    }

    bool getTail(T & val) {
        if (tail == NULL) {
            return false;
        }
        val = tail->getVal();
        return true;
    }

    bool getItem(int n, T & val) {
        ListItem<T> * it = head;
        int i = 0;
        while ((it != NULL)  && (i < n)) {
            it = it->getNext();
            i++;
        }
        
        if (it == NULL) {
            return false;
        }
        val = it->getVal();
        return true;
    }

    void clear() {
        ListItem<T> * it = head;

        while (it != NULL) {
            head = it->getNext();
            delete it;
            it = head;
        }

        head = NULL;
        tail = NULL;
        count = 0;
    }

    void add(T val) {
        if (!contains(val)) {
            addNew(val);
        }
    }

    void addNew(T val) {
        ListItem<T> * item = new ListItem<T>(val);

        if (head == NULL) {
            head = item;
            tail = item;
        } else {
            tail->setNext(item);
            tail = item;
        }
        
        count++;
    }

    void push(T val) {
        ListItem<T> * item = new ListItem<T>(val);
        
        if (head == NULL) {
            head = item;
            tail = item;
        } else {
            item->setNext(head);
            head = item;
        }
        
        count++;
    }
    
    bool pop(T & val) {
        if (head != NULL) {
            val = head->getVal();
            removeHead();
            return true;
        }
        return false;
    }
    
    void removeHead() {
        if (head != NULL) {
            ListItem<T> * headIt = head;
            
            head = headIt->getNext();
            if (head == NULL) {
                tail = NULL;
            }
            
            delete headIt;
            count--;
        }
    }
    
    void removeTail() {
        ListItem<T> * it = head;
        ListItem<T> * prevIt = NULL;

        while (it != NULL) {
            if (it->getNext() == tail) {
                if (prevIt == NULL) {
                    head = NULL;
                    tail = NULL;
                } else {
                    prevIt->setNext(NULL);
                    tail = prevIt;
                }
                
                delete it;
                count--;
                return;
            }
            
            prevIt = it;
            it = it->getNext();
        }
    }

    void remove(T val) {
        ListItem<T> * it = head;
        ListItem<T> * prevIt = NULL;

        while (it != NULL) {
            if (it->getVal() == val) {
                if (prevIt == NULL) {
                    head = it->getNext();
                    if (head == NULL) {
                        tail = NULL;
                    }
                } else if (it == tail) {
                    prevIt->setNext(NULL);
                    tail = prevIt;
                } else {
                    prevIt->setNext(it->getNext());
                }

                delete it;
                count--;
                return;
            }

            prevIt = it;
            it = it->getNext();
        }
    }

    bool contains(T val) {
        ListItem<T> * it = head;

        while (it != NULL) {
            if (it->getVal() == val) {
                return true;
            }

            it = it->getNext();
        }

        return false;
    }

    bool isEmpty() {
        return count == 0;
    }
    
    int size() {
        return count;
    }
    
    void forAll(void f(T v)) {
        ListItem<T> * it = head;
        while (it != NULL) {
            f(it->getVal());
            it = it->getNext();
        }
    }
    
    void forAll(void f(T v, void * data), void * data) {
        ListItem<T> * it = head;
        while (it != NULL) {
            f(it->getVal(), data);
            it = it->getNext();
        }
    }
    
    void copyFrom(List<T> * otherList) {
        ListItem<T> * it = otherList->head;
        while (it != NULL) {
            add(it->getVal());
            it = it->getNext();
        }
    }
    
    void moveFrom(List<T> * otherList) {
        copyFrom(otherList);
        otherList->clear();
    }
    
    void copyAllFrom(List<T> * otherList) {
        ListItem<T> * it = otherList->head;
        while (it != NULL) {
            addNew(it->getVal());
            it = it->getNext();
        }
    }
    
    void moveAllFrom(List<T> * otherList) {
        copyAllFrom(otherList);
        otherList->clear();
    }
};

#endif
