#include "Timer.h"
#include "InterruptManager.h"

Timer::Timer(byte _pin) {
    pin = _pin;
    isAttached = false;
    lastTickCount = 0;
    lastTickTimeMS = 0;
    setEnabled(false);
}
    
bool Timer::start() {
    stop();
    lastTickCount = 0;
    lastTickTimeMS = 0;
    interruptManager.attachHandlerToPin(this, pin, CHANGE);
    if (!isOk()) {
        stop();
        return false;
    }
    setEnabled(true);
    return true;
}

void Timer::stop() {
    setEnabled(false);
    if (isAttached) {
        interruptManager.detachHandlerFromPin(pin);
    }
}

bool Timer::isOk() {
    return isAttached;
}

void Timer::handleInterrupt(byte intPin) {
    lastTickCount++;
    lastTickTimeMS = millis();
}

void Timer::onAttach(byte intPin) {
    isAttached = true;
}

void Timer::onDetach(byte intPin) {
    isAttached = false;
}

long Timer::getLastTickCount() {
    return lastTickCount;
}

long Timer::getLastTickTimeMS() {
    return lastTickTimeMS;
}
